define([
  "backbone",
  "daterangepicker",
  "moment"
],function(
  Backbone,
  daterangepicker,
  moment
){
  
  /*
    Выставление ведукщих нулей
    @param Int|String
    @param Int
    @return @String
  */
  function pad(num, size) {
      var s = num+"";
      while (s.length < size) s = "0" + s;
      return s;
  }
  
  
  //~ Формат типа d/m/Y
  Date.prototype.formatDay = function () {
    return "".concat(
      this.getFullYear()
    ).concat(
      "/"
    ).concat(
      this.getMonth() + 1
    ).concat(
      "/"
    ).concat(
      this.getDate()
    ).concat(
      "/"
    )
  };
  
  
  //~ Формат типа m/Y
  Date.prototype.formatMonth = function () {
    return "".concat(
      this.getFullYear()
    ).concat(
      "/"
    ).concat(
      this.getMonth() + 1
    ).concat(
      "/"
    )
  };
  
  
  
  // Формат ymd
  Date.prototype.formatRange = function () {
    return "".concat(
      pad( this.getFullYear() , 4 )
    ).concat(
      pad( this.getMonth() + 1 , 2)
    ).concat(
      pad( this.getDate() , 2)
    )
  };
  
  //~ Формат типа m/Y
  Date.prototype.formatYear = function () {
    return "".concat(
      this.getFullYear()
    ).concat(
      "/"
    )
  };
  
  
  //~ Понедельник
  function getMonday(d) {
    d = new Date(d);
    var day = d.getDay(),
        diff = d.getDate() - day + (day == 0 ? -6:1); // adjust when day is sunday
    return new Date(d.setDate(diff));
  }
  
  
  //~ Воскресенье
  function getSunday(d){
    return new Date(d.getFullYear(), d.getMonth(), d.getDate() - d.getDay() + 7);
  }
  
  
  //~ Первый день месяца
  function firstDay( d ){
    return new Date(d.getFullYear(), d.getMonth(), d.getDate() - d.getDay() + 7);
  }
  
  //~ Последний день месяца
  function lastDay( d ){
    return new Date(d.getFullYear(), d.getMonth() + 1, 0);
  }
  
  
  
  function prevMonth( dateObj ) {
    var tempDateObj = new Date(dateObj);

    if(tempDateObj.getMonth) {
      tempDateObj.setMonth(tempDateObj.getMonth() - 1);
    } else {
      tempDateObj.setYear(tempDateObj.getYear() - 1);
      tempDateObj.setMonth(12);
    }

    return tempDateObj
  };
  
  
  

  return Backbone.View.extend({
  
  
    events : {
  
      'click' : function( e ){
        
        var myDate = new Date();

        var prevWeekDate = new Date( myDate.getTime() - 7 * 24 * 60 * 60 * 1000 )
        var prevMonthObj = prevMonth( myDate )
        var prevYearObj = new Date( myDate.setFullYear(myDate.getFullYear() - 1))
        
        
        /*
         Текущая неделя - переход на сегодня
         Предыдущая - переход на тот же день недели на той неделе
         
        */
        $( '#date-range1' ).daterangepicker({
          startOfWeek: 'monday',
          "autoApply": true,
          separator : '/',
          autoClose: false,
          time: {
            enabled: true
          },
          endDate : false,
          "locale": {
              //~ "format": "YYYY/MM/DD",
              "separator": "/",
              "applyLabel": "Применить",
              "cancelLabel": "Закрыть",
              "fromLabel": "От",
              "toLabel": "До",
              "customRangeLabel": "Вручную",
              "weekLabel": "W",
              "daysOfWeek": [
                  "Вс",
                  "Пн",
                  "Вт",
                  "Ср",
                  "Чт",
                  "Пт",
                  "Сб"
              ],
              "monthNames": [
                  "Янаварь",
                  "Февраль",
                  "Март",
                  "Апрель",
                  "Май",
                  "Июнь",
                  "Июль",
                  "Август",
                  "Сентябрь",
                  "Октябрь",
                  "Ноябрь",
                  "Декабрь"
              ],
              "firstDay": 1
          },
           minViewMode: 1,
          "maxDate": new Date().formatDay(),
          "ranges": {
            "За месяц": [
                prevWeekDate.formatMonth(),
                "empty_date"
            ],
            "За год": [
                new Date().formatYear(),
                "empty_date"
            ],
          }
        },function(start, end, label) {
          
          
          var example_loc = window.location.pathname.replace(/^\/(cmeter|meter|vmeter)\/([0-9]*)\/(.*)$/,function( _, obj, id ){
            return "/" + obj + "/" + id + "/";
          })
          
          
          if( end._d != "Invalid Date" ){
          
            // Пользователь кликнул на кадлендаре
            if( start._d instanceof Date && end._d instanceof Date ){
              
              // Нажал на один и тот же день
              if( start._d.formatDay() == end._d.formatDay() ){
                example_loc += "archive/" + start._d.formatDay()
                
              // нажал на разные дни и хочет увидеть диапазон
              }else{
                example_loc += "archive/" + start._d.formatRange() + "-" + end._d.formatRange() + "/"
              }
              
              window.location.pathname = example_loc;
              return false;
              
            }
          }
          
          
          var regexp_ar = [
            new RegExp(/^([0-9]*)\/$/),
            new RegExp(/^([0-9]*)\/([0-9]*)\/$/)
          ]
          
          var valid_start = regexp_ar.map(function( reg ){
          
            if( reg.test( start._i ) ){
              start._i.replace( reg , function( _ , year , month ){
                example_loc += "archive/" + year + "/" 
                if( month ){
                  example_loc += month + "/"
                }
              })
            }
            
            window.location.pathname = example_loc;
          
          })
          
          
          return false;
        })
        
        .trigger('click')
        
        return false;
      }
    
    },
  
    el : '#selected_date_archive',
  
  
  
  });

});