/*

  Прибор который содержится в Интерфейсе лоры

  */
function addLead(strDollar, len, leadChar = "0") {
  result = strDollar;
  while(result.length < len) {
   result = leadChar + result;
  }
  return result;
}


  var ABP_MODEL = Backbone.Model.extend({
    defaults : {
      appsKey     : "",
      devAddress  : "",
      nwksKey     : ""
    },

    validate : function(){
      if( !this.__validate_devAddress() ){
        return "invalid devAddress"
      }

      if( !this.__validate_appsKey() ){
        return "invalid appsKey"
      }

      if( !this.__validate_nwksKey() ){
        return "invalid nwksKey"
      }
    },

    initialize : function( data ){
      if( typeof data == "undefined" ){
        return 
      }
      // var val = addLead( Number(data.devAddress).toString(16) , 8)
      data.devAddress = addLead( Number(data.devAddress).toString(16) , 8)
      this.set('devAddress' , data.devAddress)
      return data
    },

    __validate_devAddress : function(){
      return this.get('devAddress').toUpperCase().match(/^([A-F|0-9]{8})$/) !== null
    },

    __validate_appsKey : function(){
      return this.get('appsKey').toUpperCase().match(/^([A-F|0-9]{32})$/) !== null
    },

    __validate_nwksKey : function(){
      return this.get('nwksKey').toUpperCase().match(/^([A-F|0-9]{32})$/) !== null
    },

  })

  var OTAA_MODEL = Backbone.Model.extend({
    defaults : {
      appEui        :"",
      appKey        :"",
      last_join_ts  : 0
    },

    validate : function(){
      if( !this.__validate_appEui() ){
        return "invalid appEui"
      }

      if( !this.__validate_appKey() ){
        return "invalid appKey"
      }
    },

    __validate_appEui : function(){
      return this.get('appEui').toUpperCase().match(/^([A-F|0-9]{16})$/) !== null
    },

    __validate_appKey : function(){
      return this.get('appKey').toUpperCase().match(/^([A-F|0-9]{32})$/) !== null
    }

  })


  var CHANELMASK_MODEL = Backbone.Model.extend({
    defaults : {
      channel10En   : false,
      channel11En   : false,
      channel12En   : false,
      channel13En   : false,
      channel14En   : false,
      channel15En   : false,
      channel16En   : false,
      channel1En    : true,
      channel2En    : true,
      channel3En    : true,
      channel4En    : true,
      channel5En    : true,
      channel6En    : true,
      channel7En    : true,
      channel8En    : true,
      channel9En    : false
    }
  })


  var FREQUENCYPLAN_MODEL = Backbone.Model.extend({
    defaults : {
      freq4 : 867100000,
      freq5 : 867300000,
      freq6 : 867500000,
      freq7 : 867700000,
      freq8 : 867900000
    },

    validate : function(){
      if( !this.__validate_freq4() ){
        return "invalid freq4"
      }
      if( !this.__validate_freq5() ){
        return "invalid freq5"
      }
      if( !this.__validate_freq6() ){
        return "invalid freq6"
      }
      if( !this.__validate_freq7() ){
        return "invalid freq7"
      }
      if( !this.__validate_freq8() ){
        return "invalid freq8"
      }
    },

    /*
      Валидация частотного плана
      Если передан параметр, то будет считаться он
      Если не передан, то будет считаться внутреннее поле.
      Сделано для проверки валидации кастомного значения
    */
    __validate_freq4 : function( value ){
      return this.__validate_freq( typeof value == "undefined" ? this.get('freq4') : value )
    },  

    __validate_freq5 : function( value ){
      return this.__validate_freq( typeof value == "undefined" ? this.get('freq5') : value )
    },

    __validate_freq6 : function( value ){
      return this.__validate_freq( typeof value == "undefined" ? this.get('freq6') : value )
    },

    __validate_freq7 : function( value ){
      return this.__validate_freq( typeof value == "undefined" ? this.get('freq7') : value )
    },

    __validate_freq8 : function( value ){
      return this.__validate_freq( typeof value == "undefined" ? this.get('freq8') : value )
    },


    __validate_freq : function( value ){
      return !isNaN( value ) && Number( value )<=1020000000 && Number( value )>=862000000 || Number(value) == 0
    },

    __is_validate_value_fom_check : function( value ){
      return  Number( value ) > 0
    }


  })

  var POSITION_MODEL = Backbone.Model.extend({
    defaults : {
      altitude  :0,
      latitude  :0,
      longitude :0
    }
  })


  var SETTINGSAPPLYSTATUS_MODEL = Backbone.Model.extend({
    defaults : {
      channelMaskApllied  : true,
      delayRx1Applied     : true,
      drRx2Applied        : true,
      freq4Applied        : true,
      freq5Applied        : true,
      freq6Applied        : true,
      freq7Applied        : true,
      freq8Applied        : true,
      freqRx2Applied      : true
    }
  })


  var LORA_DEVICE = Backbone.Model.extend({
    defaults : {
      OTAA                  : new OTAA_MODEL,
      ABP                   : new ABP_MODEL,
      adr                   : true,
      channelMask           : new CHANELMASK_MODEL,
      class                 : "CLASS_A",
      delayJoin1            : 5,
      delayJoin2            : 6,
      delayRx1              : 1,
      delayRx2              : 2,
      devEui                : "",
      devName               : "",
      drRx2                 : 0,
      fcnt_down             : 0,
      fcnt_up               : 2,
      freqRx2               : 869525000,
      frequencyPlan         : new FREQUENCYPLAN_MODEL,
      lastRssi              : -108,
      lastSnr               : 6.5,
      last_data_ts          : 1540542080649,
      position              : new POSITION_MODEL,
      preferDr              : 5,
      preferPower           : 14,
      reactionTime          : 1000,
      rxWindow              : 2,
      serverAdrEnable       : true,
      settingsApplyStatus   : new SETTINGSAPPLYSTATUS_MODEL,
      useDownlinkQueueClassC: false
    },

    initialize : function( data ){
      if( typeof data == "undefined" ){
        return
      }
      this.set('ABP',                  new ABP_MODEL(data.ABP) )
      this.set('OTAA',                 new OTAA_MODEL(data.OTAA) )
      this.set('channelMask',          new CHANELMASK_MODEL(data.channelMask) )
      this.set('frequencyPlan',        new FREQUENCYPLAN_MODEL(data.frequencyPlan) )
      this.set('position',             new POSITION_MODEL(data.position) )
      this.set('settingsApplyStatus',  new SETTINGSAPPLYSTATUS_MODEL(data.settingsApplyStatus) )
      // this.set('devEui',  data.devEui )
    },


    __toJSON : function(){
      var data = this.toJSON()
      data.ABP                  = this.get('ABP').toJSON()
      data.ABP.devAddress       = parseInt(data.ABP.devAddress ,16)
      data.OTAA                 = this.get('OTAA').toJSON()
      data.channelMask          = this.get('channelMask').toJSON()
      data.frequencyPlan        = this.get('frequencyPlan').toJSON()
      data.position             = this.get('position').toJSON()
      data.settingsApplyStatus  = this.get('settingsApplyStatus').toJSON()
      data.settingsApplyStatus  = this.get('settingsApplyStatus').toJSON()
      return data
    },


    validate : function(){

      if( this.get('ABP') instanceof ABP_MODEL  ){
        if( !this.get('ABP').isValid() ){
          return this.get('ABP').validate()
        }
      }


      if( this.get('OTAA') instanceof OTAA_MODEL  ){
        if( !this.get('OTAA').isValid() ){
          return this.get('OTAA').validate()
        }
      }

      if( !this.__validate_devEui() ){
        return "invalid devEui"
      }

      if( !this.__validate_devName() ){
        return "invalid devName"
      }

      if( this.get('preferDr') < 1 || this.get('preferDr') > 6 ){
        return "out of range preferDr"
      }

      if( [14, 11, 8, 5, 2].indexOf( this.get('preferPower') ) == -1 ){
        return "out of range preferPower"
      }

      if( this.get('rxWindow') < 1 || this.get('rxWindow') > 2 ){
        return "out of range rxWindow"
      }

      if( this.get('delayRx1') < 1 || this.get('delayRx1') > 15 ){
        return "out of range delayRx1"
      }

      if( this.get('delayJoin1') < 0 || this.get('delayJoin1') > 15 ){
        return "out of range delayJoin1"
      }

      if( Number( this.get('reactionTime') ) < 1 || String( this.get('reactionTime') ).length > 16 ){
        return "out of range reactionTime"
      }

    },


    __validate_devEui : function(){
      return this.get('devEui').match(/^([A-F|0-9]{16})$/) !== null
    },

    __validate_devName : function(){
      return this.get('devName') !== ""
    },

  });