/*
 * Модальное окно редактирования или добавления БС Лора
 */
 var LORA_EDIT_ADD_MODAL = Backbone.View.extend({

  template : "#lora_add_tpl",

  initialize : function( _model , collection_ ){
    this.model = _model;
    this.collection = collection_

    var self = this;

    // Разные сценарии для проверки соединения
    this.listenTo( this , "success_socket" , function(){
      $('#footer_modal_lora').empty().append($('#lora_test_connect_success').html())
      $('#test_auth').removeClass('btn-disabled').addClass('btn-info')
    })
    this.listenTo( this , "error_socket" , function(){
      $('#footer_modal_lora').empty().append($('#lora_test_connect_error').html())
      $('#test_auth').addClass('btn-disabled').removeClass('btn-info')
    })
    this.listenTo( this , "wait_socket" , function(){
      $('#footer_modal_lora').empty().append($('#lora_test_connect_wait').html())
    })

    // Разные сценарии для проверки авторизации
    this.listenTo( this , "success_auth" , function(){
      $('#footer_modal_lora').empty().append($('#lora_test_auth_success').html())
    })
    this.listenTo( this , "error_auth" , function(){
      $('#footer_modal_lora').empty().append($('#lora_test_auth_error').html())
    })


    this.listenTo( this.model , "change" , this.change_btn_status )

    this.render();
  },

  // Выставление статусов кнопкам Сохранения и проверки соидинения
  change_btn_status : function(){
    if( this.model.isValid() ){
      $('#save').removeClass('btn-disabled').addClass('btn-success')
    }else{
      $('#save').addClass('btn-disabled').removeClass('btn-success')
    }

    if( this.model.isValid() ){
      $('#test').removeClass("btn-disabled").addClass('btn-info')
    }else{
      $('#test').addClass("btn-disabled").removeClass('btn-info')
    }

    // когда пользователь меняет данные, мы не можем быть уверены
    // что он подключится к БС
    // Пусть перепроверяет
    if( typeof ( this.model.changed.ip || this.model.changed.port ) != "undefined" ){
      $('#footer_modal_lora').empty()
      $('#test_auth').addClass('btn-disabled').removeClass('btn-info')
    }

  },

  
  render : function(){
    var tpl_c = _.template( $(this.template).html() );
    this.$el = $( tpl_c({ lora_model : this.model }) ).modal('show');
    var self = this;
    $( this.$el ).on('shown.bs.modal' , function(){
      new lora_ip_label_view( self.model )
      new lora_port_label_view( self.model )

      self.change_btn_status();
    }).on('hidden.bs.modal' , function(){
      $(this).remove();
    })
    
  },
  
  events : {

    "keyup #ip_lora" : function( e ){
      this.model.set('ip' , $(e.target).val() );
    },

    "keyup #port_lora" : function( e ){
      this.model.set('port' , $(e.target).val() );
    },

    "keyup #login_lora" : function( e ){
      this.model.set('login' , $(e.target).val() );
    },

    "keyup #pass_lora" : function( e ){
      this.model.set('pass' , $(e.target).val() );
    },

    "keyup #about_lora" : function( e ){
      this.model.set('about' , $(e.target).val() );
    },

    // test connected
    "click #test" : function(){

      var self = this;

      try{
        var websocket = new WebSocket("ws://" + this.model.get('ip') + ":" + this.model.get('port') );
      }catch{
        self.trigger("error_socket");
      }
      websocket.onopen = function(evt) {
        if( evt instanceof Event ){
          self.trigger("success_socket");
          websocket.close();
        }
      };
      websocket.onerror = function(evt) {
        if( evt instanceof Event ){
          self.trigger("error_socket");
        }
      };

      self.trigger("wait_socket");

      websocket.onopen();

    },


    // test auth
    "click #test_auth" : function(){

      var self = this;

      var websocket = new WebSocket("ws://" + this.model.get('ip') + ":" + this.model.get('port') );
      websocket.onopen = function(evt) {
        if( evt instanceof Event ){
          self.trigger("wait_socket");
          websocket.send(self.model.__auth());
        }
      };
      websocket.onmessage = function( data ){

        if( typeof (data.data && JSON.parse(data.data).status) != "undefined" ){
          status = JSON.parse(data.data).status
          if( status == "true" ){
            self.trigger("success_auth");
          }else{
            self.trigger("error_auth");
          }

          websocket.close();
        }
      }

      websocket.onopen();

    },


    "click #save" : function(){
      var self = this;
      this.model.save(null , {
        success : function( new_model ){
          $(self.$el).modal('hide');
          if( self.collection){
            self.collection.fetch();
          }
        }
      });
    }
    
  }
  
});




/*
  Вид отвечает за вывод валидности поля IP адреса 
  */
  var lora_ip_label_view = Backbone.View.extend({

    template : "#ip_label_tpl",

    el : "#ip_label",

  // Вводил ли пользователь значение
  keyup_input : false,
  
  initialize : function( _model ){
    this.model = _model;
    this.listenTo( this.model , "change:ip" , function(){
      this.keyup_input = true
    })
    this.listenTo( this.model , "change:ip" , this.render )
    this.render()
  },
  
  render : function(){

    var tpl_c = _.template( $(this.template).html() )
    
    var invalid = false;
    
    if( this.keyup_input ){
      if( typeof this.model.is_valid_ip() != "undefined" ){
        invalid = true;
      }
    }

    $(this.$el).empty().append( 
      tpl_c({ 
        invalid : invalid , 
        reason : invalid ? this.model.is_valid_ip() : "" 
      }) 
      )
  }
});



/*
  Вид отвечает за вывод валидности поля Порта 
  */
  var lora_port_label_view = Backbone.View.extend({

    template : "#port_label_tpl",

    el : "#port_label",

  // Вводил ли пользователь значение
  keyup_input : false,
  
  initialize : function( _model ){
    this.model = _model;
    this.listenTo( this.model , "change:port" , function(){
      this.keyup_input = true
    })
    this.listenTo( this.model , "change:port" , this.render )
    this.render()
  },
  
  render : function(){

    var tpl_c = _.template( $(this.template).html() )
    
    var invalid = false;
    
    if( this.keyup_input ){
      if( typeof this.model.is_valid_port() != "undefined" ){
        invalid = true;
      }
    }

    $(this.$el).empty().append( 
      tpl_c({ 
        invalid : invalid , 
        reason : invalid ? this.model.is_valid_port() : "" 
      }) 
      )
  }
});


