/*
  
  Модальное окно для редактирования прибора учета Лора

*/
var METER_MODAL_EDIT = Backbone.View.extend({

  template: "#lora_counter_add_edit",

  initialize: function (_model, _model_types, _lora_model, _lora_types, _base_station, _lora_device_types) {

    this.model = _model
    this.model_types = _model_types
    this.lora_model = _lora_model
    this.lora_types = _lora_types
    this.base_station = _base_station
    this.lora_device_types = _lora_device_types


    var self = this

    this.model.set('converter_id', _base_station.get('id'))

    this.listenTo(this.lora_model, "change", this.__rerender_btn)

    this.base_station.socket.addEventListener('message', function (event) {

      if (typeof (event.data) != "undefined") {
        resp = JSON.parse(event.data)
        if (typeof resp.cmd != "undefined") {

          if (resp.cmd == "manage_devices_resp") {
            var lora_add_elem = resp.device_add_status.shift()

            if (typeof lora_add_elem != "undefined") {
              switch (lora_add_elem.status) {
                case "added":
                  $(self.$el).modal('hide')
                  break;
                case "updated":
                  $(self.$el).modal('hide')
                  break;
                default:
                  $('#error_save').slideUp(0).slideDown(500)
                  switch (lora_add_elem.status) {
                    case "emptyDataForUpdate":
                      $('#error_save').text("Нет данных для изменений")
                      break;
                    case "noRegisterKeys":
                      $('#error_save').text("Устройство еще не существует")
                      break;
                    case "repetitionDevAddr":
                      $('#error_save').text("Устройство с соответствующим «devAddress» уже зарегистрировано на сервере")
                      break;
                    case "abpReginfoAlreadyExist":
                      $('#error_save').text("Информация активации ABP для соответствующего устройства уже существует на сервере")
                      break;
                    case "otaaReginfoAlreadyExist":
                      $('#error_save').text("Информация активации OTAA для соответствующего устройства уже существует на сервере")
                      break;
                    case "reginfoAlreadyExist":
                      $('#error_save').text("Регистрационная информация для соответствующего устройства уже существует на сервере")
                      break;
                    case "maxDevCountReached":
                      $('#error_save').text("Максимальное ограничение количества устройств достигнуто")
                      break;
                    case "nothingToUpdate":
                      $('#error_save').text("Изменения из существующих параметров устройства не обнаружены")
                      break;
                    case "updateViaMacBuffer":
                      $('#error_save').text("Полученные параметры должны обновляться с помощью команд MAC, и их нельзя применить немедленно")
                      break;
                  }
                  break;
              }
            }

          } else if (resp.cmd == "get_device_appdata_resp") {

            var dev_app = self.lora_device_types.findWhere({ devEui: self.lora_model.get('devEui') })
            dev_app.set('device_type', self.model.get('type_string'))
            self.base_station.save_type_device(dev_app)

          }

        }

      }

    })


    this.render();
  },

  /*
    
    Перерисовываем кнопку сохранения

  */
  __rerender_btn: function () {


    this.model.set('type_string', this.lora_types.findWhere({ id: Number($('#changedType').val()) }).get('name'))
    console.log( this.lora_model.validate() )
    if (this.lora_model.isValid()) {
      $("#save").removeClass('btn-disabled').addClass('btn-success')
    } else {
      $("#save").addClass('btn-disabled').removeClass('btn-success')
    }
  },

  render: function () {
    var tpl_c = _.template($(this.template).html())
    this.$el = $(tpl_c({ lora_model: this.model })).modal('show')

    var self = this;

    $(this.$el).on('hidden.bs.modal', function () {
      $(this).remove();
    }).on('shown.bs.modal', function () {

      new METER_ACTIVATION_BY_PERS(self.model, self.lora_model)
      new METER_OVER_THE_AIR(self.model, self.lora_model)
      new METER_MAIN_SETTINGS(self.model, self.lora_model, self.lora_types, self.lora_device_types)
      new METER_REGIONAL_SETTINGS(self.lora_model)
      new METER_RX_SETTINGS(self.lora_model)
      new ADAPTIVE_SETTINS(self.lora_model)
      new CLASS_C_SETTINGS(self.lora_model)
      self.__rerender_btn();
    })
  },

  expert_sessings_contoller: false,


  events: {
    "click #save": function () {
      var self = this;
      var is_add = this.model.get('id') == 0
      this.model.save(null, {
        success: function () {
          if (is_add) {
            self.base_station.add_counter(self.lora_model)
          } else {
            self.base_station.save_counter(self.lora_model)
          }
        }
      })
    },

    "click #expert_sessings_contoller": function () {
      this.expert_sessings_contoller = !this.expert_sessings_contoller
      var action = this.expert_sessings_contoller ? "slideDown" : "slideUp"
      $("#expert_sessings")[action](500)
      if (this.expert_sessings_contoller) {
        $("#expert_sessings_contoller").find('.fa-chevron-circle-down')
          .removeClass('fa-chevron-circle-down')
          .addClass('fa-chevron-circle-up')
      } else {
        $("#expert_sessings_contoller").find('.fa-chevron-circle-up')
          .removeClass('fa-chevron-circle-up')
          .addClass('fa-chevron-circle-down')
      }
    }
  }

})



/*
  
  Вид отвечает за отображение и валидации блока 
  Activation by personalisation (ABP)

  @param {Model} Модель реального прибора у нас в системе
  @param {Model} Модель прибора лоры

*/
var METER_ACTIVATION_BY_PERS = Backbone.View.extend({
  el: "#activation_by_pers",

  template: "#activation_by_pers_tpl",

  initialize: function (_model, _lora_model) {
    this.model = _model
    this.lora_model = _lora_model

    this.listenTo(this.lora_model.get('ABP'), "change", function () {

      var self = this;

      var error_has = function (id, is_sucess) {
        if (!is_sucess) {
          $(self.$el).closest('.form-group').addClass('has-error')
          $(id).addClass('text-danger')
        } else {
          $(self.$el).closest('.form-group').removeClass('has-error')
          $(id).removeClass('text-danger')
        }
      }
      error_has('#devAddr_label', self.lora_model.get('ABP').__validate_devAddress());
      error_has('#appsKey_label', self.lora_model.get('ABP').__validate_appsKey());
      error_has('#nwksKey_label', self.lora_model.get('ABP').__validate_nwksKey());

    });

    this.render()
  },

  render: function () {
    var tpl_c = _.template($(this.template).html())
    $(this.$el).empty().append(tpl_c({ device: this.lora_model }))
    $("#devAddr_input").focus()
  },

  events: {

    "keyup #devAddr_input": function (e) {
      this.__e_devAddr_input(e)
    },

    "blur #devAddr_input": function (e) {
      this.__e_devAddr_input(e)
    },

    "keyup #appsKey_input": function (e) {
      this.__e_appsKey_input(e)
    },

    "blur #appsKey_input": function (e) {
      this.__e_appsKey_input(e)
    },

    "keyup #nwksKey_input": function (e) {
      this.__e_nwksKey_input(e)
    },

    "blur #nwksKey_input": function (e) {
      this.__e_nwksKey_input(e)
    }

  },

  __e_devAddr_input: function (e) {
    this.lora_model.get('ABP').set('devAddress', $(e.target).val().toUpperCase())
  },

  __e_appsKey_input: function (e) {
    this.lora_model.get('ABP').set('appsKey', $(e.target).val().toUpperCase())
  },

  __e_nwksKey_input: function (e) {
    this.lora_model.get('ABP').set('nwksKey', $(e.target).val().toUpperCase())
  },

})




var METER_OVER_THE_AIR = Backbone.View.extend({
  el: "#over_the_air",

  template: "#over_the_air_tpl",

  initialize: function (_model, _lora_model) {
    this.model = _model
    this.lora_model = _lora_model

    this.listenTo(this.lora_model, "change", function () {

      var self = this;

      var error_has = function (id, is_sucess) {
        if (!is_sucess) {
          $(self.$el).closest('.form-group').addClass('has-error')
          $(id).addClass('text-danger')
        } else {
          $(self.$el).closest('.form-group').removeClass('has-error')
          $(id).removeClass('text-danger')
        }
      }

      error_has('#AppEUI_label', self.lora_model.get('OTAA').__validate_appEui());
      error_has('#AppKey_label', self.lora_model.get('OTAA').__validate_appKey());

    });

    this.render()
  },

  render: function () {
    var tpl_c = _.template($(this.template).html())
    $(this.$el).empty().append(tpl_c({ device: this.lora_model }))
  },



  events: {
    "keyup #AppEUI_input": function (e) {
      this.lora_model.get('OTAA').set('appEui', $(e.target).val().toUpperCase())
      this.lora_model.trigger('change')
    },
    "keyup #AppKey_input": function (e) {
      this.lora_model.get('OTAA').set('appKey', $(e.target).val().toUpperCase())
      this.lora_model.trigger('change')
    },
  }
})


var METER_MAIN_SETTINGS = Backbone.View.extend({
  el: "#main_settings",

  template: "#main_settings_tpl",

  initialize: function (_model, _lora_model, _lora_types, _lora_devices_type) {
    this.model = _model
    this.lora_model = _lora_model
    this.lora_types = _lora_types
    this.lora_devices_types = _lora_devices_type

    this.listenTo(this.lora_model, "change", function () {

      var self = this;

      var error_has = function (id, is_sucess) {
        if (!is_sucess) {
          $(self.$el).closest('.form-group').addClass('has-error')
          $(id).addClass('text-danger')
        } else {
          $(self.$el).closest('.form-group').removeClass('has-error')
          $(id).removeClass('text-danger')
        }
      }

      error_has('#DevName_label', self.lora_model.__validate_devName());
      error_has('#DevEUI_label', self.lora_model.__validate_devEui());

    });

    this.render()
  },

  render: function () {
    var tpl_c = _.template($(this.template).html())
    $(this.$el).empty().append(tpl_c({
      device: this.lora_model,
      lora_types: this.lora_types.toArray(),
      model: this.model,
      lora_device_types: this.lora_devices_types
    }))
  },

  events: {

    "keyup #DevName_input": function (e) {
      this.lora_model.set('devName', $(e.target).val())
      this.model.set('about', $(e.target).val())
      this.lora_model.trigger('change')
    },

    "keyup #DevEUI_input": function (e) {
      this.lora_model.set('devEui', $(e.target).val().toUpperCase())
      this.model.set('num_serial', $(e.target).val())
      this.lora_model.trigger('change')
    },

    "change #changedClass": function (e) {
      this.lora_model.set('class', $(e.target).val().toUpperCase())
      this.lora_model.trigger('change')
    },

    "change #changedType": function (e) {
      this.model.set('type_string', this.lora_types.findWhere({ id: Number($(e.target).val()) }).get('name'))
    }

  }
})




var METER_REGIONAL_SETTINGS = Backbone.View.extend({
  el: "#regional_settings",

  template: "#regional_settings_tpl",

  eu: new FREQUENCYPLAN_MODEL({
    freq4: 867100000,
    freq5: 867300000,
    freq6: 867500000,
    freq7: 867700000,
    freq8: 867900000
  }),

  ru: new FREQUENCYPLAN_MODEL({
    freq4: 864100000,
    freq5: 864300000,
    freq6: 868800000,
    freq7: 0,
    freq8: 0
  }),

  is_custom: false,

  initialize: function (_lora_model) {
    this.model = _lora_model
    this.model.get('frequencyPlan').set(this.eu.toJSON())
    this.render()
  },

  render: function () {
    var tpl_c = _.template($(this.template).html())
    $(this.$el).empty().append(tpl_c({
      device: this.model,
      ru: this.ru,
      eu: this.eu,
      is_custom: this.is_custom
    }))


    var keys = Object.keys(this.model.get('frequencyPlan').attributes)
    for (var i = 0; i <= keys.length - 1; i++) {
      this.node_event_valid(keys[i])
    }
  },

  events: {
    "change #regional_patern": function (e) {
      switch (Number($(e.target).val())) {
        case 1:
          this.model.get('frequencyPlan').set(this.eu.toJSON())
          this.model.set("freqRx2", 869525000)
          break;
        case 2:
          this.model.get('frequencyPlan').set(this.ru.toJSON())
          this.model.set("freqRx2", 869050000)
          break;
      }
      this.is_custom = Number($(e.target).val()) == 3
      this.render()
    },

    "keyup #input_freq4": function (e) {
      this.model.get('frequencyPlan').set('freq4', $(e.target).val())
      this.node_event_valid('freq4')
    },

    "keyup #input_freq5": function (e) {
      this.model.get('frequencyPlan').set('freq5', $(e.target).val())
      this.node_event_valid('freq5')
    },

    "keyup #input_freq6": function (e) {
      this.model.get('frequencyPlan').set('freq6', $(e.target).val())
      this.node_event_valid('freq6')
    },

    "keyup #input_freq7": function (e) {
      this.model.get('frequencyPlan').set('freq7', $(e.target).val())
      this.node_event_valid('freq7')
    },

    "keyup #input_freq8": function (e) {
      this.model.get('frequencyPlan').set('freq8', $(e.target).val())
      this.node_event_valid('freq8')
    },
  },


  // Выставление валидации для ноды
  node_event_valid: function (freq) {
    $('#div_' + freq)[!this.model.get('frequencyPlan')["__validate_" + freq]() ? "addClass" : "removeClass"]('has-error')
    $('#check_' + freq).prop("checked", this.model.get('frequencyPlan').__is_validate_value_fom_check(this.model.get('frequencyPlan').get(freq)))
  }


})


var METER_RX_SETTINGS = Backbone.View.extend({

  el: "#device_rx_settings",

  template: "#device_rx_settings_tpl",

  initialize: function (_lora_model) {

    this.model = _lora_model
    this.render()

  },

  render: function () {
    var tpl_c = _.template($(this.template).html())
    $(this.$el).empty().append(tpl_c({
      lora_device: this.model
    }))
  },


  events: {
    "change #delayRx1": function (e) {
      this.model.set('delayRx1', Number($(e.target).val()))
    },

    "change #rx_window": function (e) {
      this.model.set('rxWindow', Number($(e.target).val()))
    },

    "change #delayRx2": function (e) {
      this.model.set('drRx2', Number($(e.target).val()))
    },

    "change #delayJoin1": function (e) {
      this.model.set('delayJoin1', Number($(e.target).val()))
    },
  }

})



var ADAPTIVE_SETTINS = Backbone.View.extend({

  el: "#adaptive_settings",

  template: "#adaptive_settings_tpl",

  initialize: function (lora_model_) {
    this.model = lora_model_
    this.listenTo(this.model, "change:serverAdrEnable", function () {
      $("#preferred_transmit_power").attr("disabled", !this.model.get('serverAdrEnable'))
      $("#preferred_data_rate").attr("disabled", !this.model.get('serverAdrEnable'))
    })
    this.render()
  },

  render: function () {
    var tpl_c = _.template($(this.template).html())
    $(this.$el).empty().append(tpl_c({ lora_device: this.model }))
  },

  events: {
    "change #enable_server_adr": function (e) {
      this.model.set('serverAdrEnable', $(e.target).prop('checked'))
    },

    "change #preferred_data_rate": function (e) {
      this.model.set('preferDr', Number($(e.target).val()))
    },

    "change #preferred_transmit_power": function (e) {
      this.model.set('preferPower', Number($(e.target).val()))
    }

  }

})


var CLASS_C_SETTINGS = Backbone.View.extend({

  el: "#class_c_settings",

  template: "#class_c_settings_tpl",

  initialize: function (lora_model_) {
    this.model = lora_model_
    this.listenTo(this.model, "change:class", this.render)
    this.render()
  },

  render: function () {
    var tpl_c = _.template($(this.template).html())
    $(this.$el).empty().append(tpl_c({ lora_device: this.model }))
  },

  events: {
    "keyup #reactionTime": function (e) {
      this.model.set('reactionTime', Number($(e.target).val()))
    },

    "change #useDownlinkQueueClassC": function (e) {
      this.model.set('useDownlinkQueueClassC', $(e.target).prop('checked'))
    }
  }

})

