/*
  
  Этот вид полность отвечвет за модальное окно

*/


var LORA_SYNC_DEVICES_EACH_ROW =  Backbone.View.extend({


  template : "#tbody_unsync_devices_tpl",

  initialize : function( parent ){

    this.parent = parent
    this.$el = $(parent.$el).find('tbody')
    var self = this;

    this.listenTo( parent.collection     , "add remove"         , this.render )
    this.listenTo( parent.rm_collection  , "add remove"         , this.render )
    this.listenTo( parent.checked        , "add remove"         , this.render )
    this.listenTo( parent.meter_types    , "change:device_type" , this.render )

    $(this.$el).on('change' , "input" , function( e ){
      self.checked_input( e )
    })
    $(this.$el).on('change' , "select" , function( e ){
      self.checked_select( e )
    })

    this.render()
  },

  render : function(){

    var tpl_c = _.template( $(this.template).html() )
    if( typeof this.collection != "undefined" ){
      $(this.$el).empty().append( 
        tpl_c({ 
          devices       : this.parent.collection.toArray() , 
          rm_devices    : this.parent.rm_collection.toArray() , 
          device_types  : this.parent.meter_types,
          types         : this.parent.types,
          checked       : this.parent.checked,
        }) 
      )
    }
  },


  checked_input : function( e ){
    var cid       = $(e.target).closest('tr').attr('id')
    var device    = this.parent.collection.get( cid )
    var rm_device = this.parent.rm_collection.get( cid )

    var method = $(e.target).prop('checked') ? "add" : "remove"

    if( typeof device != "undefined" ){
      this.parent.checked[method]( device )
    }else if( typeof rm_device != "undefined" ){
      this.parent.checked[method]( rm_device )
    }

    $('#all').prop('checked' , false)
  },

  checked_select : function( e ){
    var cid = $(e.target).closest('tr').attr('id')
    var device = this.parent.collection.get( cid )
    var meter_type = this.parent.meter_types.findWhere( { devEui : device.get('devEui') } )
    meter_type.set("device_type" ,  $(e.target).val() == "0" ? "" : $(e.target).val() )
  }

});


/*
    
  Модальное окно для синхронихзации приборо
  
  @param _unsync_devices  {Collection} Все приборы, которые надо синхронизировать
  @param _types           {Collection} Все типы приборов
  @param _meter_types     {Collection} Все связи приборов с типами
  @param _base_station    {Model}      Модель БС

*/
var LORA_SYNC_DEVICES = Backbone.View.extend({

  collection  : new LORA_DEVICES(),
  model       : new LORA_CONVERTER(),
  types       : new LORA_TYPES(),
  meter_types : new LORA_DEVICE_TYPES(),

  // Приборы, которые выделили для синхронизации
  checked     : new LORA_DEVICES(),


  template : "#undefined_device_modal_tpl",

  initialize : function( _unsync_devices , _remove_device , _types , _meter_types , _base_station , _meters ){

    if( typeof ( _unsync_devices || _remove_device || _types || _meter_types || _base_station || _meters ) == "undefined" ){
      return
    }

    var self = this;

    this.collection     = _unsync_devices
    this.rm_collection  = _remove_device
    this.model          = _base_station
    this.types          = _types
    this.meters         = _meters
    this.meter_types    = _meter_types

    // При изменении статусов выделения для синхронизации - надо перерисовывать кнопку
    // для начала синхронизации
    this.listenTo( this.checked , "add remove" , function(){
      if( this.checked.length > 0 ){
        $("#run_sync").removeClass('btn-disabled').addClass('btn-success')
      }else{
        $("#run_sync").addClass('btn-disabled').removeClass('btn-success')
      }
    })

    // При старте - все приборы не выделенные
    _.each( this.collection.toArray() , function( _device ){
      _device.set({
        sync : false,
        sync_status : 0
      })
    })

    _.each( this.rm_collection.toArray() , function( _device ){
      _device.set({
        sync : false,
        sync_status : 0
      })
    })


    this.render()
  },


  // Проверка надо скрывать или нет модальное окно
  // Надо скрывать по заверешнии работы
  __is_hide_modal : function(){
    if( this.collection.where({ sync : true }).length == 0 && 
      this.rm_collection.where({ sync : true }).length == 0 ){
      $(this.$el).modal('hide')
      this.model.trigger('all_devices_sync')
    }
  },


  render : function( ){

    var tpl_c = _.template( $(this.template).html() )
    var self = this;
    this.$el = $( tpl_c() ).modal('show')

    $(this.$el).on('hidden.bs.modal' , function(){
      $(this).remove()
      self.meters.fetch({ url : "/api/?object=meter&filter=converter:" + self.model.get('id') })
    }).on( 'shown.bs.modal' , function(){
      new LORA_SYNC_DEVICES_EACH_ROW( self ) 
    })

  },

  events : {
    "click #run_sync" : function(){

      var self = this;

      // Ждем окончания синхронизации
      this.listenTo( this.checked , "remove" , function(){
        if( !self.checked.length ){
          $(self.$el).modal('hide')
        }
      })

      _.each( this.checked.toArray() , function( device ){

        var device = device
        var devEui = device.get('devEui')

        var is_removed = typeof self.rm_collection.findWhere({ devEui : devEui }) != "undefined"
        var is_added = typeof self.collection.findWhere({ devEui : devEui }) != "undefined"


        if( is_added ){
          var metertype = self.meter_types.findWhere({ devEui : device.get('devEui') });
          var type = self.types.findWhere({ id : Number( metertype.get('device_type') ) })
          var new_model = new METER_MODEL({
            converter_id  : self.model.get('id'),
            num_serial    : device.get('devEui'),
            about         : device.get('devName'),
            type_string   : type.get('name')
          });

          console.log( new_model )


          new_model.save(null , {success : function( _model ){
            self.checked.remove(device)
            self.collection.remove(device)
          } , error : function(){
            console.log(111)
          }})

        }else if( is_removed ){

          device.destroy({success : function(){
            self.checked.remove(device)
            self.rm_collection.remove(device)
          }})

        }



      })

      $("#all").prop('checked' , false)
    },

    "change #all" : function( e ){

      var method = $(e.target).prop('checked') ? "add" : "remove"

      for (var i = this.collection.length - 1; i >= 0; i--) {
        var device = this.collection.at( i )

        if( this.meter_types.findWhere({ devEui : device.get('devEui') }).get('device_type') == "" ){
          continue
        }
        this.checked[method]( device )
      }

      for (var i = this.rm_collection.length - 1; i >= 0; i--) {
        var device = this.rm_collection.at( i )
        this.checked[method]( device )
      }
    }
  }

})